# Kickstart to build Rocky 8 image for Raspberry Pi 4 hardware (aarch64)
#

# (pull from pgreco:  https://github.com/psgreco/sig-core-AltArch/tree/master/image_build


# Disk setup
clearpart --initlabel --all
part /boot --asprimary --fstype=vfat --size=300 --label=boot
part swap --asprimary --fstype=swap --size=512 --label=swap
part / --asprimary --fstype=ext4 --size=2800 --label=RPIROOT



# Repos setup:
repo --name="BaseOS"     --baseurl=http://download.rockylinux.org/pub/rocky/8/BaseOS/aarch64/os/ --cost=100
repo --name="AppStream"  --baseurl=http://download.rockylinux.org/pub/rocky/8/AppStream/aarch64/os/ --cost=200 --install
repo --name="PowerTools" --baseurl=http://download.rockylinux.org/pub/rocky/8/BaseOS/aarch64/os/ --cost=300 --install
# Rocky Rpi kernel repo, we need a more permanent place for this:
repo --name="Rpi-Kernel" --baseurl=http://skip.linuxdn.org/RockyRpi/Rpi-Kernel/ --cost=20



# Install proces:
install
keyboard us --xlayouts=us --vckeymap=us
rootpw --lock
# FIXME user creation here does not work ?
# user --name="rocky" --password="rockylinux" --plaintext --gecos="Rocky default user" --groups=wheel --uid=1000
timezone --isUtc --nontp UTC
selinux --enforcing
firewall --enabled --port=22:tcp
network --bootproto=dhcp --device=link --activate --onboot=on
services --enabled=sshd,NetworkManager,chronyd
shutdown
bootloader --location=none
lang en_US.UTF-8



# Package selection:
%packages
@core
chrony
cloud-utils-growpart
net-tools
NetworkManager-wifi
vim
bash-completion
nano


raspberrypi2-firmware
raspberrypi2-kernel4
aarch64-img-extra-config

%end


# Post install scripts:
%post


# Write initial boot line to cmdline.txt (we will update the root partuuid further down)
cat > /boot/cmdline.txt << EOF
console=ttyAMA0,115200 console=tty1 root= rootfstype=ext4 elevator=deadline rootwait
EOF

# Run this once to fix the wifi:
sed -i  's/boardflags3=0x48200100/boardflags3=0x44200100/g'  /lib/firmware/brcm/brcmfmac43455-sdio.raspberrypi,4-model-b.txt


# Apparently kickstart user was not working, attempt to do it here?
/sbin/useradd -c "Rocky Linux default user" -G wheel -m -U rocky
echo "rockylinux" | passwd --stdin  rocky

# Need to write several files to help with various things here.
# First, the all-important README :

cat >/home/rocky/README << EOF
== Rocky 8 Raspberry Pi Image (experimental) ==

This is a minimal Rocky 8 install intended for Raspberry Pi 4 devices (architecture is aarch64).

It may work on a Pi 3 , but that has not been tested.  We are looking for Rpi 3 owners to test if you'd like to try it.

This image WILL NOT WORK on a Raspberry Pi 1 or 2, we are 64-bit only, and have no support for 32-bit ARM processors.  Sorry :-/.


IMAGE NOTES / DIFFERENCES FROM STOCK ROCKY 8:
  
  - Based on Rocky Linux 8.4, points to production Rocky 8 aarch64 repositories
  - Has an additional repository that contains kernel packages for Raspberry Pi
  - Includes script that fixes the wifi.  Simple edit of a txt firmware settings file.  Will need to be run whenever linux-firmware gets upgraded
  - Includes @minimal-install , plus a few quality of life packages like vim, bash-completion, etc.
  - Initial User "rocky" (default password: "rockylinux").  Root password disabled, rocky user is a sudoer
  - Partitions are 300 MB /boot , 512 MB swap, 2800 MB rootfs.  Requires a 4 GB or larger storage device to serve as your disk



If you want to automatically resize your / partition, just type the following (as root user):
sudo rootfs-expand

It should fill your main rootfs partition to the end of the disk.

EOF


# Next, script to fix the wifi firmware setting:
cat >/home/rocky/fix-wifi-rpi4.sh << 'EOF'
#!/bin/bash

# Simple hacky fix to correct wifi settings on the Rpi4 so it loads properly:

sed -i  's/boardflags3=0x48200100/boardflags3=0x44200100/g'  /lib/firmware/brcm/brcmfmac43455-sdio.raspberrypi,4-model-b.txt

echo "Patched /lib/firmware/brcm/brcmfmac43455-sdio.raspberrypi,4-model-b.txt .  You can reboot and wifi should work, or do:    rmmod brcmfmac;  modprobe brcmfmac;  "

EOF

chmod 755 /home/rocky/fix-wifi-rpi4.sh

chown -R rocky:rocky /home/rocky


# Add rpi-kernel repo:
cat >/etc/yum.repos.d/Rocky_Rpi_kernel.repo << 'EOF'
# Raspberry Pi Kernel repository
#
# This is an additional repository that hosts special kernel builds to use for the Raspberry Pi
# Currently in an unsigned, temporary location - should change soon!
#
# Kernel upstream source is pulled from:  https://git.centos.org/rpms/raspberrypi2
#

[Rocky_Rpi_Kernel]
name=Rocky Linux 8 Raspberry Pi Kernel
baseurl=http://skip.linuxdn.org/RockyRpi/Rpi-Kernel/
gpgcheck=0
enabled=1

EOF

# Cleanup before shipping an image

# Remove ifcfg-link on pre generated images
rm -f /etc/sysconfig/network-scripts/ifcfg-link

# Remove machine-id on pre generated images
rm -f /etc/machine-id
touch /etc/machine-id

# Ensure no ssh keys are present
rm -f /etc/ssh/*_key*

# Clean yum cache
yum clean all

%end

# Add the PARTUUID of the rootfs partition to the kernel command line
%post --nochroot

# Extract the UUID of the rootfs partition from /etc/fstab
UUID_ROOTFS="$(/bin/cat $INSTALL_ROOT/etc/fstab | \
/bin/awk -F'[ =]' '/\/ / {print $2}')"

# Get the PARTUUID of the rootfs partition
PART_UUID_ROOTFS="$(/sbin/blkid  "$(/sbin/blkid --uuid $UUID_ROOTFS)" | \
/bin/awk '{print $NF}' | /bin/tr -d '"' )"

# Configure the kernel commandline
/bin/sed -i "s/root= /root=${PART_UUID_ROOTFS} /" $INSTALL_ROOT/boot/cmdline.txt
echo "cmdline.txt looks like this, please review:"
/bin/cat $INSTALL_ROOT/boot/cmdline.txt

%end
